/*
 *	W_VT52.H
 *	Windowed Terminal Emulator
 */

/*
 *	This header file contains all prototypes, definitions, etc. for the
 *	W_VT52.LIB (BareBones) routines.
 *
 *	This library is a useable subset of my windowed VT52 library. It
 *	provides a very simple and easy to use set of routines to ease
 *	printing to a GEM window. 
 *
 *	This library while being very flexible does not wrap all GEM window
 *	calls, thus adding another level to them which slows them down a tad
 *	and introduces the need to insure you don't call GEM window commands
 *	directly. This library is small and fairly quick. As a subset of a
 *	larger library it has been tailored to cater to text output and does
 *	not support a visable cursor. The coordinates are kept internally and
 *	can be changed via a w_escf() call. The upshoot of this is that this
 *	small library will work fine except in the case of windowed text
 *	editing in which you need a cursor.  A little ingenuity however could
 *	solve this problem as would using the larger library.
 *
 *	The workhorse of this library is the w_printf() call.  This call works
 *	exactly like C's printf() call. In fact the function uses the vsprintf()
 *	call internally. C's printf() is a very versitile function and many
 *	programmers (me included) really like its power and flexibility. Now you
 *	can have that power and flexibility designed to work within the confines
 *	of a GEM window! WOW!
 *
 */

#include <aes.h>
#include <vdi.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/*
 *	These structures are declared internally by the library and should not
 *	be referenced by the user program.
 *
 */

typedef struct _WTE_VARS
{

	short
		tw,							/* text width in pixels */
		th,							/* text height in pixels */
		rows,						/* number of rows */
		cols,						/* number of columns */
		crow,						/* current row */
		ccol;						/* current column */

} WTE_VARS;


typedef struct _WTES
{

	int
		windh;					/* window handle */
	short
		vdih,						/* VDI_Workstation struct */
		wx,							/* x position of work area */
		wy,							/* y position of work area */
		ww,							/* width of work area */
		wh;							/* height of work area */

	WTE_VARS
		vt;							/* WTE_SETTINGS struct */

} WTES;


/* W_VT52.LIB Prototypes */


/*
 *	Function Prototypes
 */

int w_init(int windh, short *vdih, short *trows, short *tcols);
/*
 *	Called after opening a window in which you wish to print out text.
 *	Pass it the window handle of the opened window and it will return
 *	the vdi handle which is important for setting the clipping rectangle
 *	in your window redraw function. This is pretty much all you should
 *	use the VDI handle for. Also you can perform a check to insure that
 *	none of the vdi handles are duplicates, I guess that this is possible
 *	on some versions of the OS. You should also insure that after you are
 *	done redrawing a window to turn the clipping rectangle off or it will
 *	mess up your w_printf(). This call also returns the total number of
 *	rows and columns are available in the window. As a rule you should
 *	insure that you have room for 2 rows and 2 columns. the w_calc() call
 *	is VERY helpful to determine the exact width and height of the work
 *	area needed for x number of rows and columns. (see w_calc()) This
 *	routine returns R_OK if ok else returns R_ERR. Up to 8 windows can
 *	be initialized. A window can only be initialized once.
 *
 */

int w_term(int windh);
/*
 *	This routines only real purpose is to release the VDI handle allocated
 *	on a previous w_init() call and do minimal clean up on the WTES array
 *	so its ready for any new w_init() calls. Returns R_OK if ok else R_ERR
 *
 */

int w_calc(int windh, short rows, short cols, short *width, short *height);
/*
 *	This is an extreamely useful function. It was added as an afterthought.
 *	With this function its possible to determine the width and height of the
 *	work area needed for x number of rows and columns.
 *
 *	windh  - windows handle (must have called w_init() successfully first.)
 *	rows   - desired number of rows.
 *	cols   - desired number of columns.
 *	width  - returned width of rectangle needed.
 *	height - returned height of rectangle needed.
 *
 *	returns: R_OK if ok, else R_ERR.
 *
 *	As a rule you should request at least 2 rows and 2 columns. Given the
 *	width and height you can then use wind_calc() to find the total size
 *	the window needs to be opened at.
 *
 */
 
int w_escf(int windh, int com, short *row, short *col);
/*
 *	This function processes or VT52 type escape functions. This is the
 *	only way to make use of the escape functions. I had thought about
 *	Parsing the w_printf() string to look for esc functions but not only
 *	would this be a pain it would also slow it down some more.
 *
 *	windh   - the windows handle (must have used w_init() first)
 *	com     - escape command. (use given defines)
 *	row     - depends on function used.
 *	column  - depends on function used.
 *
 *	returns: R_OK if ok, else R_ERR.
 *
 */

int w_gcor(int windh, short *row, short *col);
/*
 *	This function will return the current position of the cursor.
 *
 *	windh   - the windows handle (must have used w_init() first)
 *	row     - filled with current row number
 *	col     - filled with current column number
 *
 */

int w_printf(int windh, const char *fmt, ...);
/*
 *	Used to print text to an initialized window.
 *
 *	windh		- the windows handle (must have used w_init() first)
 *	fmt			- a pointer to the format string
 *  ...			- variable number of arguments.
 *
 *	returns: R_OK if ok, else R_ERR.
 *
 *	Works exactly like C's printf() call. In fact as stated it uses
 *	vsprintf() internally.  This function uses the v_gtext() call.
 *	It understands \r and \n (CR or Newline respectively) and will
 *	output all characters there is graphics for. ie) tabs and other
 *	control characters. If you were doing windowed editing you'd
 *	handle these keys as you see fit.
 *
 *	It is important to note that to increase speed this function does
 *	not check to see if the window has been topped or maybe has changed
 *	places and things like that. This routine does not set or use the
 *	VDI clipping rectangle. To insure proper output in a background
 *	window you must set the VDI clipping rectangle yourself or use a
 *	WM_REDRAW routine and generate a redraw message for the window
 *	manually.  This is a bit of a pain but allows for greater flexibility
 *	speed and overall programmers comfort (thats me) :) If you set the
 *	clipping rectangle you must insure you turn it off after you are
 *	done otherwise it could screw up w_printf()'s printing (by clipping
 *	its output.) Also if the window has been moved or resized you must
 *	inform the library of this through the w_resize() call. Again because
 *	to increase speed the library does not check before every print. Imagine
 *	how slow it'd be if printing a character at a time!?
 *
 */

int w_resize(int windh, short *trow, short *tcol);
/*
 *	This function is used to inform the library that a window has been
 *	resized or moved. You should insure that the window remains completely
 *	on screen if any printing is going to be done to it otherwise you may
 *	experience some graphics droppings. This is because the library doesn't
 *	use clipping internally for stuff like blits and rectangle fills and
 *	the like. The function is very easy to use though and should be a part
 *	of your WM_MOVED & WM_RESIZED handling functions. This will insure that
 *	all internals such as the windows work area and total number of rows
 *	and columns gets updated correctly.
 *
 *	windh		- windh of window in question.
 *	trows   - new total number of rows available (if resized)
 *	tcols		- new total number of columns available (if resized)
 *
 *	returns: R_OK if ok, else R_ERR.
 *
 */


/*
 *	The following functions are used by the above routines they should not
 *	be called by the users programs directly.  This is implied as they all
 *	take a pointer to a WTES structure. An array of 8 WTES structures is
 *	declared internally by w_init().
 *
 */

int w_pcur(WTES *wtes, int com);
int w_blit(WTES *wtes, int sx, int sy, int sw, int sh, int dx, int dy, int dw, int dh);
int w_fill(WTES *wtes, int x, int y, int w, int h);
int w_setc(WTES *wtes, short *row, short *col);
int w_outs(WTES *wtes, char *line);
int w_puts(WTES *wtes, char *buff);


/*
 * Windowed Terminal Emulator Definitions
 */


/* return values for ALL functions */

#define			R_OK			1				/* function returns normally */
#define			R_ERR			0				/* function returns with error */

/* Needed Stuff */

#define			WTESIZ		8							/* number of WTES sturctures */
#define			WTESL			(WTESIZ-1)		/* Index Number of last WTES struct */

/* esc commands used by THIS library */

#define			ESC_A			1				/* cursor up */
#define			ESC_B			2				/* cursor down */
#define			ESC_C			3				/* cursor right */
#define			ESC_D			4				/* cursor left */
#define			ESC_E			5				/* clear screen & home cursor */
#define			ESC_H			6				/* home cursor */
#define			ESC_I			7				/* cursor up (with scroll if nec.) */
#define			ESC_Y			12			/* position cursor at row, column */


/* external declaration */

/*
 *	Again user should not reference this array of WTES structs.
 *
 */

extern		WTES wtes[];				/* array of 8 WTES structs */


/*	MISCELLANEOUS NOTES
 *
 *	Use of the library is quite simple. #include this header file in
 *	any of your sources that calls library functions.
 *
 *	When linking include the name of the library after all your other
 *	libraries.
 *
 *	Confine windows to the desktops work area.
 *
 *	Do printing to background windows via your programs WM_REDRAW function,
 *	set the clipping rectangle accordingly and be sure to turn off clipping
 *	as soon as you are done.
 *
 *	Do not manipulate the WTES array (structures) directly.
 *	Do not modify any VDI attributes directly (use w_escf())
 *
 *	Using w_init(), w_resize(), w_printf() and w_term() you can easily
 *	incorperate versitile printing to windows into new or even existing
 *	programs. And this will not dramatically increase the size of your
 *	compile program. Each function in the library has been compiled on
 *	lattice C 5.60 with global optimization for time & speed.
 *
 *	Some common sequences:
 *
 *	Create window with max size set to desktops work area (window 0)
 *	Use w_calc() to get width & height of needed work area
 *	Use wind_calc() to determine full size of window.
 *	Insure window is confined to desktop (may be redundant)
 *	Open window
 *	Call w_init() with windows handle.
 *	Insure window is topped for printing.
 *	Use w_escf() and w_printf() for printing to window.
 *	Use normal GEM bindings for manipulating windows (ie; setting title, etc)
 *	If window not topped use WM_REDRAW routine for printing.
 *	If window resized or moved call w_resize() to update internals.
 *	done? Close window, call w_term(), delete window.
 *	Thats pretty much it.
 *
 *	Most importantly explore, experiment, be creative, and have fun.
 *
 */


/*
 * eof
 */
